/*
DeltaChars Menu

Get ready for some copy-pasting from DialogBox.lua...

v1.1
*/

local DC = DeltaChars
local DB = DC.DialogueBox
local Kris = DC.Kris
local Ralsei = DC.Ralsei
local AA = DC.AxAngle

local DM = {}
DeltaChars.Menu = DM

//Save all the save-able stuff
DM.save = function(player)
	DC.saveDialogue(player)
	DC.SwapSave(player)
	DC.saveColors(player)
	
	Kris.SlideSave(player)
	Ralsei.HatSave(player)
	if player == consoleplayer or player == secondarydisplayplayer
		local value = player[AA.PLAYER_VARIABLE]
		AA.WriteToFile(value, player == secondarydisplayplayer and 2 or 1)
	end
end

DM.fun = function(player, gamemode)
	if gamemode then return not (gametyperules & GTR_FRIENDLY) end
	return multiplayer and not (gametyperules & GTR_FRIENDLY)
	and (player == server or IsPlayerAdmin(player))
end

//Button to text conversion
DM.buttonConv = {
	[BT_WEAPONMASK] = "None",
	[BT_CUSTOM1] = "C1",
	[BT_CUSTOM2] = "C2",
	[BT_CUSTOM3] = "C3",
	[BT_ATTACK] = "Fire",
	[BT_FIRENORMAL] = "Fire N",
	[BT_WEAPONNEXT] = "W Next",
	[BT_WEAPONPREV] = "W Prev",
	[BT_TOSSFLAG] = "T Flag",
}

//Button order
DM.buttons1 = {
	BT_WEAPONMASK,
	BT_CUSTOM1,
	BT_CUSTOM2,
	BT_CUSTOM3,
	BT_ATTACK,
	BT_FIRENORMAL,
	BT_WEAPONNEXT,
	BT_WEAPONPREV,
	BT_TOSSFLAG,
}

DM.buttons2 = {}
for i, button in ipairs(DM.buttons1)
	DM.buttons2[button] = i
end

//Get a line of text graphics for use by the script
DM.getText = function(text)
	local letters = {}
	local string = text
	local x = 0
	local y = 0
	
	//Get the letters for the text
	local color = DB.TEXT_COLORS[1]
	for i = 1, #text
		// snip the first character from the string
		local char = string:sub(1, 1)
		local nextChar = string:sub(2, 2)
		
		string = $:sub(2)
		
		// special case: newline
		if char == '\n'
			x = 0
			y = $ + DB.LINE_BREAK
			continue
		end
		
		// special case: space
		if char == ' '
			x = $ + DB.SPACE
			continue
		end
		
		if not DB.CharIsVisible(char)
			local byte = char:byte()
			
			// special case: colors
			if byte >= 128 and byte <= 143
				color = DB.TEXT_COLORS[byte - 127]
			end
			
			i = $ - 1 // does not count as a character
			continue
		end
		
		// add the character to the draw list
		table.insert(letters, {
			x = x,
			y = y,
			char = char,
			color = color
		})
		x = $ + DB.SPACE
	end
	return letters
end

//Draw the specified set of text graphics
DM.drawText = function(v, x, y, text, flags)
	local letters = DM.getText(text)
	for i, letter in ipairs(letters)
		if not letter.patch
			letter.patch = DB.GetCharPatch(v, letter.char)
			letter.colormap = v.getColormap(TC_DEFAULT, letter.color)
		end
		v.drawScaled(x + letter.x, y + letter.y, FRACUNIT, letter.patch, flags, letter.colormap)
	end
end

//Define each page
DM.PAGE_CONFIG = 1 << 0
DM.PAGE_KRIS = 1 << 1
DM.PAGE_SUSIE = 1 << 2
DM.PAGE_RALSEI = 1 << 3
DM.PAGE_KRISCOLOR = 1 << 4
DM.PAGE_SUSIECOLOR = 1 << 5
DM.PAGE_RALSEICOLOR = 1 << 6

DM.SKINS = {
	[DM.PAGE_KRIS] = "Kris",
	[DM.PAGE_SUSIE] = "Susie",
	[DM.PAGE_RALSEI] = "Ralsei",
	[DM.PAGE_KRISCOLOR] = "Kris",
	[DM.PAGE_SUSIECOLOR] = "Susie",
	[DM.PAGE_RALSEICOLOR] = "Ralsei",
}

DM.PAGEMOVE = {
	[DM.PAGE_KRIS] = {DM.PAGE_SUSIE, DM.PAGE_RALSEI},
	[DM.PAGE_SUSIE] = {DM.PAGE_RALSEI, DM.PAGE_KRIS},
	[DM.PAGE_RALSEI] = {DM.PAGE_KRIS, DM.PAGE_SUSIE},
}

DM.PAGECOLOR = {
	[DM.PAGE_KRIS] = DM.PAGE_KRISCOLOR,
	[DM.PAGE_SUSIE] = DM.PAGE_SUSIECOLOR,
	[DM.PAGE_RALSEI] = DM.PAGE_RALSEICOLOR,
	[DM.PAGE_KRISCOLOR] = DM.PAGE_KRIS,
	[DM.PAGE_SUSIECOLOR] = DM.PAGE_SUSIE,
	[DM.PAGE_RALSEICOLOR] = DM.PAGE_RALSEI,
}

//Define color junk
DM.COLORROWS = 3
DM.COLORCOLUMNS = 7
DM.COLORX = 54*FRACUNIT
DM.COLORY = 118*FRACUNIT
DM.COLORXDIST = 35*FRACUNIT
DM.COLORYDIST = 26*FRACUNIT

DM.numColors = 0
DM.getSelectColors = function(index)
	if DM.numColors != #skincolors or DM.selectColors == nil
		DM.numColors = #skincolors
		DM.selectColors = {}
		local gottenColors = {}
		local color = SKINCOLOR_WHITE
		for i = 1, #skincolors
			if skincolors[color] and skincolors[color].accessible
				table.insert(DM.selectColors, color)
				gottenColors[color] = true
			end
			color = M_GetColorAfter($)
			if not color or gottenColors[color]
				break
			end
		end
	end
	
	if index
		return DM.selectColors[index] or SKINCOLOR_NONE
	else
		return DM.selectColors
	end
end
DM.getSelectColors()

//Define each page's selections
DM.pageSelections = {}

DM.pageSelections[DM.PAGE_CONFIG] = {
	{
		text = "Char Options",
		selectfunc = function(player)
			player.dc_menusel = 1
			player.dc_menupage = DM.PAGE_KRIS
		end,
	},
	{
		text = "Dialogue",
		value = function(player)
			return DC.dialogue.value and "ON" or "OFF"
		end,
		selectfunc = function(player)
			if player == consoleplayer or player == secondarydisplayplayer
				if DC.dialogue.value
					CV_StealthSet(DC.dialogue, 0)
				else
					CV_StealthSet(DC.dialogue, 1)
				end
			end
		end,
	},
	{
		text = function(player)
			if DM.fun(player)
				return "Fun"
			else
				return "\134Fun"
			end
		end,
		value = function(player)
			if DM.fun(player)
				return DC.fun.value and "ON" or "OFF"
			else
				return (DC.fun.value or not DM.fun(player, true)) and "\134ON" or "\134OFF"
			end
		end,
		selectfunc = function(player)
			if DM.fun(player)
				if netgame
					CV_AddValue(DC.fun, 1)
				elseif DC.fun.value
					CV_StealthSet(DC.fun, 0)
				else
					CV_StealthSet(DC.fun, 1)
				end
			else
				return false
			end
		end,
	},
	{
		text = "Switch Next",
		value = function(player)
			return DM.buttonConv[player.dc_swapnext] or "C1"
		end,
		selectfunc = function(player)
			local num = DM.buttons2[player.dc_swapnext or BT_CUSTOM1]
			local button = DM.buttons1[num + 1]
			local left = player.dc_menuside < 0 or player.dc_menuspin == 1
			if left
				button = DM.buttons1[num - 1]
			end
			
			if not button
				button = left and DM.buttons1[#DM.buttons1]
				or BT_WEAPONMASK
			end
			DC.SwapNext(player, tostring(button), true)
		end,
		side = true,
	},
	{
		text = "Switch Prev",
		value = function(player)
			return DM.buttonConv[player.dc_swapprev] or "C3"
		end,
		selectfunc = function(player)
			local num = DM.buttons2[player.dc_swapprev or BT_CUSTOM3]
			local button = DM.buttons1[num + 1]
			local left = player.dc_menuside < 0 or player.dc_menuspin == 1
			if left
				button = DM.buttons1[num - 1]
			end
			
			if not button
				button = left and DM.buttons1[#DM.buttons1]
				or BT_WEAPONMASK
			end
			DC.SwapPrev(player, tostring(button), true)
		end,
		side = true,
	},
	{
		text = "Dance",
		value = function(player)
			return DM.buttonConv[player.dc_dance] or "C2"
		end,
		selectfunc = function(player)
			local num = DM.buttons2[player.dc_dance or BT_CUSTOM2]
			local button = DM.buttons1[num + 1]
			local left = player.dc_menuside < 0 or player.dc_menuspin == 1
			if left
				button = DM.buttons1[num - 1]
			end
			
			if not button
				button = left and DM.buttons1[#DM.buttons1]
				or BT_WEAPONMASK
			end
			DC.Dance(player, tostring(button), true)
		end,
		side = true,
	},
	{
		text = "Back",
		selectfunc = function(player)
			player.dc_menu = false
			DM.save(player)
		end,
	},
	x = 85*FRACUNIT, y = 63*FRACUNIT,
	vx = 130*FRACUNIT, vy = DB.LINE_BREAK,
}

DM.MPColors = {
	text = "MP Colors",
	value = function(player)
		return player.dc_mpcolor and "ON" or "OFF"
	end,
	selectfunc = function(player)
		if player.dc_mpcolor == 1
			DC.mpcolor(player, "off", true)
		else
			DC.mpcolor(player, "on", true)
		end
	end,
}

DM.Skincolor = {
	text = "Skincolor",
	value = function(player)
		return R_GetNameByColor(DC.getColor(player, DM.SKINS[player.dc_menupage]:lower(), true)):upper()
	end,
	selectfunc = function(player)
		player.dc_menupage = DM.PAGECOLOR[$] or $
		
		local skincolor = DC.getColor(player, DM.SKINS[player.dc_menupage]:lower(), true)
		local colors = DM.getSelectColors()
		player.dc_menusel = 1
		for i, color in ipairs(colors)
			if color == skincolor
				player.dc_menusel = i
				break
			end
		end
	end,
}

DM.pageSelections[DM.PAGE_KRIS] = {
	DM.MPColors,
	DM.Skincolor,
	{
		text = "Cam Slide",
		value = function(player)
			return player.kriscameraslide and "ON" or "OFF"
		end,
		selectfunc = function(player)
			if player.kriscameraslide
				Kris.SlideCommand(player, "off", true)
			else
				Kris.SlideCommand(player, "on", true)
			end
		end,
	},
	{
		text = function(player)
			if DM.fun(player)
				return "Dance Slide"
			else
				return "\134Dance Slide"
			end
		end,
		value = function(player)
			if DM.fun(player)
				return DC.krisDanceSlide.value and "ON" or "OFF"
			else
				return (DC.krisDanceSlide.value or not DM.fun(player, true)) and "\134ON" or "\134OFF"
			end
		end,
		selectfunc = function(player)
			if DM.fun(player)
				if netgame
					CV_AddValue(DC.krisDanceSlide, 1)
				elseif DC.krisDanceSlide.value
					CV_StealthSet(DC.krisDanceSlide, 0)
				else
					CV_StealthSet(DC.krisDanceSlide, 1)
				end
			else
				return false
			end
		end,
	},
	{
		text = "\134Guts",
		value = 1,
	},
	{
		text = "Back",
		selectfunc = function(player)
			player.dc_menusel = 1
			player.dc_menupage = DM.PAGE_CONFIG
		end,
	},
	x = 50*FRACUNIT, y = 102*FRACUNIT,
	vx = 130*FRACUNIT, vy = 13*FRACUNIT,
}

DM.pageSelections[DM.PAGE_SUSIE] = {
	DM.MPColors,
	DM.Skincolor,
	{
		text = "Cam Ground Ax",
		value = function(player)
			return (player[AA.PLAYER_VARIABLE] & AA.GROUND) and "ON" or "OFF"
		end,
		selectfunc = function(player)
			AA.Command(player, tostring(player[AA.PLAYER_VARIABLE] ^^ AA.GROUND),
			"-SILENT", "-RAW", "-NOSAVE")
		end,
	},
	{
		text = "Cam Air Ax",
		value = function(player)
			return (player[AA.PLAYER_VARIABLE] & AA.AIR) and "ON" or "OFF"
		end,
		selectfunc = function(player)
			AA.Command(player, tostring(player[AA.PLAYER_VARIABLE] ^^ AA.AIR),
			"-SILENT", "-RAW", "-NOSAVE")
		end,
	},
	{
		text = "Cam Buster",
		value = function(player)
			return (player[AA.PLAYER_VARIABLE] & AA.BUSTER) and "ON" or "OFF"
		end,
		selectfunc = function(player)
			AA.Command(player, tostring(player[AA.PLAYER_VARIABLE] ^^ AA.BUSTER),
			"-SILENT", "-RAW", "-NOSAVE")
		end,
	},
	{
		text = "Back",
		selectfunc = function(player)
			player.dc_menusel = 1
			player.dc_menupage = DM.PAGE_CONFIG
		end,
	},
	x = 50*FRACUNIT, y = 102*FRACUNIT,
	vx = 130*FRACUNIT, vy = 13*FRACUNIT,
}

DM.pageSelections[DM.PAGE_RALSEI] = {
	DM.MPColors,
	DM.Skincolor,
	{
		text = "Hat",
		value = function(player)
			return player.ralseihat and "ON" or "OFF"
		end,
		selectfunc = function(player)
			if player.ralseihat
				Ralsei.HatCommand(player, "off", true)
			else
				Ralsei.HatCommand(player, "on", true)
			end
		end,
	},
	{
		text = "\134Dogness",
		value = function(player)
			local dogness = 1
			local gems = emeralds
			if gametyperules & GTR_POWERSTONES
				gems = player.powers[pw_emeralds]
			end
			
			for i = 0, 7
				if gems & (1 << i)
					dogness = $ + 1
				end
			end
			return "\134" + dogness
		end,
	},
	{
		text = "\134Fluffiness",
		value = 2,
	},
	{
		text = "Back",
		selectfunc = function(player)
			player.dc_menusel = 1
			player.dc_menupage = DM.PAGE_CONFIG
		end,
	},
	x = 50*FRACUNIT, y = 102*FRACUNIT,
	vx = 130*FRACUNIT, vy = 13*FRACUNIT,
}

//Set the number of selections for each page
for i, selections in pairs(DM.pageSelections)
	local num = 0
	for j, selection in ipairs(selections)
		num = $ + 1
	end
	selections.num = num
end

//Define page functions
DM.pageFunctions = {}

//Config thinker
table.insert(DM.pageFunctions, {
	pages = DM.PAGE_CONFIG,
	drawer = function(v, player, flags)
		//Draw background and title
		v.draw(160, 100, v.cachePatch("DR_CONFIGMENU"), flags)
		DM.drawText(v, 135*FRACUNIT, 38*FRACUNIT, "CONFIG", flags)
	end,
})

//Character Page thinker
table.insert(DM.pageFunctions, {
	pages = DM.PAGE_KRIS | DM.PAGE_SUSIE | DM.PAGE_RALSEI
	| DM.PAGE_KRISCOLOR | DM.PAGE_SUSIECOLOR | DM.PAGE_RALSEICOLOR,
	thinker = function(player)
		//Move our cursor left or right
		local side = DM.pageSelections[player.dc_menupage]
		side = $ and $[player.dc_menusel]
		side = $ and $.side
		if DM.PAGEMOVE[player.dc_menupage] and (abs(player.dc_menuside) == 1
		or (abs(player.dc_menuside) >= 15 and player.dc_menuside & 1))
		and not side
			if player.dc_menuside > 0
				player.dc_menupage = DM.PAGEMOVE[$][1]
			else
				player.dc_menupage = DM.PAGEMOVE[$][2]
			end
			S_StartSound(nil, sfx_dcmenu, player)
		end
	end,
	drawer = function(v, player, flags)
		//Draw background and title
		local skin = DM.SKINS[player.dc_menupage]
		v.draw(160, 100,
		v.cachePatch(player.dc_menupage >= DM.PAGE_KRISCOLOR and "DR_COLORMENU" or "DR_CHARMENU"),
		flags)
		
		if not skin then return end
		DM.drawText(v, 65*FRACUNIT, 40*FRACUNIT, skin, flags)
		skin = $:lower()
		
		//Draw the skin walking in the top right
		local sprite2 = player.dc_menupose and SPR2_TRNS or SPR2_WALK
		local frame = player.dc_menupose and G or (player.dc_menutime/4) % 6
		local rotation = ((player.dc_menutime/4/6) % 8) + 1
		
		local patch, flipped = v.getSprite2Patch(skin, sprite2, player.ralseihat == 1, frame, rotation)
		flipped = $ and V_FLIP or 0
		local color = DC.getColor(player, skin, true)
		
		//Draw the outline
		local colormap = v.getColormap(TC_ALLWHITE, SKINCOLOR_DC_SPARE)
		for i = -2, 2
			for j = -2, 2
				v.drawScaled(212*FRACUNIT + i*INT16_MAX, 76*FRACUNIT + j*INT16_MAX,
				INT16_MAX, patch, flags|flipped, colormap)
			end
		end
		
		//Draw the player
		v.drawScaled(212*FRACUNIT, 76*FRACUNIT,
		INT16_MAX, patch, flags|flipped, v.getColormap(skin, color))
		
		//Draw the selectable characters in the top left
		local x = 56
		local y = 75
		for i, member in ipairs(DC.skins)
			local myflags = flags
			//If selected, draw the cursor above this character
			if skin == member
				local patch = "DR_HEART" + (((player.dc_menutime/2) % 4) + 1)
				local side = DM.pageSelections[player.dc_menupage]
				side = $ and $[player.dc_menusel]
				side = $ and $.side
				if side or not DM.PAGEMOVE[player.dc_menupage]
					patch = "DR_HEART5"
				end
				v.drawScaled(x*FRACUNIT, (y - 22)*FRACUNIT, INT16_MAX,
				v.cachePatch(patch), myflags)
				
			//If not, make transparent to grey out
			else
				local trans = myflags & V_ALPHAMASK
				trans = min(($ >> FF_TRANSSHIFT) + 5, 9) << FF_TRANSSHIFT
				myflags = ($ &~ V_ALPHAMASK) | trans
			end
			
			//Draw the character
			patch = "DR_" + member:upper()
			if member == "ralsei" and player.ralseihat
				patch = $ + 2
			end
			
			v.draw(x, y, v.cachePatch(patch), myflags)
			x = $ + 26
		end
	end,
})

//Color Page thinker
//tfw I created a menu selection system and I can't even use it for this
table.insert(DM.pageFunctions, {
	pages = DM.PAGE_KRISCOLOR | DM.PAGE_SUSIECOLOR | DM.PAGE_RALSEICOLOR,
	thinker = function(player)
		local colors = DM.getSelectColors()
		local sound = false
		
		//Exit
		if player.dc_menuspin == 1
			player.dc_menupage = DM.PAGECOLOR[$] or $
			player.dc_menusel = 2
			
			S_StartSound(nil, sfx_dcselc, player)
			return
		end
		
		//Move our cursor up or down
		local maxselect = #colors + DM.COLORCOLUMNS - (#colors % DM.COLORCOLUMNS) + 1
		if (abs(player.dc_menuforward) == 1
		or (abs(player.dc_menuforward) >= 15 and player.dc_menuforward & 1))
			if player.dc_menuforward < 0
				player.dc_menusel = $ + DM.COLORCOLUMNS
				if player.dc_menusel >= maxselect
					player.dc_menusel = $ - maxselect + 1
				end
			else
				player.dc_menusel = $ - DM.COLORCOLUMNS
				if player.dc_menusel <= 0
					player.dc_menusel = $ + maxselect - 1
				end
			end
			sound = true
		end
		
		//Move our cursor left or right
		if (abs(player.dc_menuside) == 1
		or (abs(player.dc_menuside) >= 15 and player.dc_menuside & 1))
			if player.dc_menuside > 0
				player.dc_menusel = $ + 1
				if (player.dc_menusel % DM.COLORCOLUMNS) == 1
					player.dc_menusel = $ - DM.COLORCOLUMNS
				end
			else
				player.dc_menusel = $ - 1
				if not (player.dc_menusel % DM.COLORCOLUMNS)
					player.dc_menusel = $ + DM.COLORCOLUMNS
				end
			end
			sound = true
		end
		
		if sound then S_StartSound(nil, sfx_dcmenu, player) end
		
		//Select a color
		if player.dc_menujump == 1
			local color = colors[player.dc_menusel]
			if not color
				S_StartSound(nil, sfx_dclose, player)
				return
			end
			
			local skin = DM.SKINS[player.dc_menupage]:lower()
			player["dc_"..skin.."color"] = color
			DC.setColor(player)
			
			player.dc_menupose = TICRATE/3
			S_StartSound(nil, sfx_dcselc, player)
		end
	end,
	drawer = function(v, player, flags)
		local x = DM.COLORX
		local y = DM.COLORY
		local rows = DM.COLORROWS
		local columns = DM.COLORCOLUMNS
		
		local colors = DM.getSelectColors()
		local color = SKINCOLOR_WHITE
		local patch = v.cachePatch("DR_COLOR")
		
		local offset = (((player.dc_menusel - 1)/columns) * columns) - columns
		offset = min(max($, 0), #colors + (#colors % columns))
		
		//Draw each color
		for i = 1, rows*columns
			local sel = i + offset
			color = colors[sel]
			if sel == player.dc_menusel
				if color
					v.drawScaled(x, y, FRACUNIT, v.cachePatch("DR_COLOR2"),
					flags, v.getColormap(TC_DEFAULT, color))
				end
				
				local string = "DR_HEART" + (((player.dc_menutime/2) % 4) + 1)
				v.drawScaled(x, y - patch.height/3*FRACUNIT, INT16_MAX, v.cachePatch(string), flags)
			elseif color
				v.drawScaled(x, y, FRACUNIT, patch,
				flags, v.getColormap(TC_DEFAULT, color))
			end
			
			if not (i % columns)
				x = DM.COLORX
				y = $ + DM.COLORYDIST
			else
				x = $ + DM.COLORXDIST
			end
		end
		
		//Draw color name
		color = colors[player.dc_menusel]
		color = $ and skincolors[$].name:upper() or "-"
		local length = (DB.SPACE/FRACUNIT)*(#color + 1) + 2
		v.drawFill(64 - 6, 78, length, 9, 31)
		DM.drawText(v, 64*FRACUNIT, 87*FRACUNIT, color, flags)
	end,
})

//Main functions for all the pages with normal selections
table.insert(DM.pageFunctions, {
	pages = DM.PAGE_CONFIG | DM.PAGE_KRIS | DM.PAGE_SUSIE | DM.PAGE_RALSEI,
	thinker = function(player)
		local selections = DM.pageSelections[player.dc_menupage]
		if not selections then return end
		
		//Move our cursor up or down
		if (abs(player.dc_menuforward) == 1
		or (abs(player.dc_menuforward) >= 15 and player.dc_menuforward & 1))
			if player.dc_menuforward < 0
				player.dc_menusel = $ + 1
				if player.dc_menusel > selections.num
					player.dc_menusel = 1
				end
			else
				player.dc_menusel = $ - 1
				if player.dc_menusel <= 0
					player.dc_menusel = selections.num
				end
			end
			S_StartSound(nil, sfx_dcmenu, player)
		end
		
		//Select a selection
		local selection = selections[player.dc_menusel]
		if player.dc_menujump == 1 or player.dc_menuspin == 1
		or (selection.side and (abs(player.dc_menuside) == 1
		or (abs(player.dc_menuside) >= 15 and player.dc_menuside & 1)))
			if player.dc_menuspin == 1 and player.dc_menusel != selections.num
				player.dc_menusel = selections.num
				S_StartSound(nil, sfx_dcmenu, player)
			elseif selection.selectfunc
				local result = selection.selectfunc(player)
				if result == false
					S_StartSound(nil, sfx_dclose, player)
				elseif result
					S_StartSound(nil, result, player)
				else
					S_StartSound(nil, sfx_dcselc, player)
				end
			else
				S_StartSound(nil, sfx_dclose, player)
			end
		end
	end,
	drawer = function(v, player, flags)
		//Draw all the possible selections, and a cursor on which one is selected
		local selections = DM.pageSelections[player.dc_menupage]
		if not selections then return end
		
		local x = selections.x
		local y = selections.y
		local vx = selections.vx
		local vy = selections.vy
		for i, selection in ipairs(selections)
			//Draw main text
			if selection.text
				if type(selection.text) == "function"
					DM.drawText(v, x, y, selection.text(player), flags)
				else
					DM.drawText(v, x, y, selection.text, flags)
				end
			end
			
			//Draw selection value
			if selection.value
				if type(selection.value) == "number"
					v.drawScaled(x + vx, y, FRACUNIT,
					v.cachePatch(selection.value == 2 and "DR_FLUFF" or "DR_GUTS"),
					flags, v.getColormap(TC_DEFAULT, SKINCOLOR_GREY))
				elseif type(selection.value) == "function"
					DM.drawText(v, x + vx, y, selection.value(player), flags)
				else
					DM.drawText(v, x + vx, y, selection.value, flags)
				end
			end
			
			//Draw the cursor if this is the selected selection
			if i == player.dc_menusel
				local patch = "DR_HEART5"
				if selection.side
					patch = "DR_HEART" + (((player.dc_menutime/2) % 4) + 1)
				end
				v.drawScaled(x - DB.SPACE, y, INT16_MAX, v.cachePatch(patch), flags)
			end
			
			//Move down a line
			y = $ + vy
		end
	end,
})

function DM.reset(player)
	player.dc_menutime = 0
	player.dc_menusel = 1
	player.dc_menupage = DM.PAGE_CONFIG
	
	player.dc_menujump = player.cmd.buttons & BT_JUMP
	player.dc_menuspin = player.cmd.buttons & (BT_SPIN|BT_CUSTOM1)
	player.dc_menuforward = 0
	player.dc_menuside = 0
	
	player.dc_menupose = 0
end

//dc_menu command
function DM.command(player, arg)
	local mo = player.mo
	if modeattacking or titlemapinaction or tutorialmode or player.playerstate
	or (mo and mo.valid and not mo.health)
		CONS_Printf(player, "You can't enter the DeltaChars menu right now.")
		return
	end
	
	if (player == consoleplayer or player == secondarydisplayplayer)
	and not CV_FindVar("showhud").value
		COM_BufInsertText(player, "showhud on")
	end
	
	player.dc_menu = not $
	if player.dc_menu
		DM.reset(player)
		S_StartSound(nil, sfx_dcselc, player)
	else
		DM.save(player)
		S_StartSound(nil, sfx_dcselc, player)
	end
end
COM_AddCommand("dc_menu", DM.command)
COM_AddCommand("dc_menu2", DM.command, COM_SPLITSCREEN)

//Menu Thinker
function DM.MenuThinker()
	for player in players.iterate
		//Intialize
		if player.dc_menutime == nil
			DM.reset(player)
			
		//Main code
		elseif player.dc_menu and not player.playerstate
		and not (mo and mo.valid and not mo.health)
			player.dc_menutime = $ + 1
			
			//Button detection
			if player.cmd.buttons & BT_JUMP
				player.dc_menujump = $ + 1
			else
				player.dc_menujump = 0
			end
			if player.cmd.buttons & (BT_SPIN|BT_CUSTOM1)
				player.dc_menuspin = $ + 1
			else
				player.dc_menuspin = 0
			end
			
			//Prevent movement
			local forwardmove = player.cmd.forwardmove
			local sidemove = player.cmd.sidemove
			player.cmd.forwardmove = 0
			player.cmd.sidemove = 0
			player.cmd.buttons = 0
			
			//Movement detection
			if abs(forwardmove) >= 15
				player.dc_menuforward = abs($) + 1
				if forwardmove < 0
					player.dc_menuforward = -$
				end
			else
				player.dc_menuforward = 0
			end
			if abs(sidemove) >= 15
				player.dc_menuside = abs($) + 1
				if sidemove < 0
					player.dc_menuside = -$
				end
			else
				player.dc_menuside = 0
			end
			
			//Pose timer
			if player.dc_menupose
				player.dc_menupose = $ - 1
			end
			
			//Run thinkers
			local currentpage = player.dc_menupage
			for i, functions in ipairs(DM.pageFunctions)
				if functions.thinker
				and (not functions.pages or (functions.pages & currentpage))
					functions.thinker(player)
				end
			end
			
		else
			//Fade away
			if player.dc_menutime
				player.dc_menutime = min($ - 1, 9)
				if player.dc_menu
					player.dc_menu = false
				end
			end
			
			//Retain lack of jump and spin buttons after menu close
			if player.dc_menujump
				if player.cmd.buttons & BT_JUMP
					player.cmd.buttons = $ &~ BT_JUMP
				else
					player.dc_menujump = 0
				end
			end
			
			if player.dc_menuspin
				if player.cmd.buttons & (BT_SPIN|BT_CUSTOM1)
					player.cmd.buttons = $ &~ (BT_SPIN|BT_CUSTOM1)
				else
					player.dc_menuspin = 0
				end
			end
		end
	end
end

//Menu HUD drawer
function DM.MenuHUD(v, player)
	if not player.dc_menutime then return end
	local menupage = player.dc_menupage
	local flags = min(max((10 - player.dc_menutime) << FF_TRANSSHIFT, 0), V_90TRANS) | V_PERPLAYER
	
	//Run drawers
	for i, functions in ipairs(DM.pageFunctions)
		if functions.drawer
		and (not functions.pages or (functions.pages & player.dc_menupage))
			functions.drawer(v, player, flags)
		end
	end
end

DC.AddHook("PreThinkFrame", DM.MenuThinker)
hud.add(DM.MenuHUD)